package at.xaxa.ledger.ui.entry.add

import android.util.Log
import androidx.compose.foundation.layout.Box
import androidx.compose.foundation.layout.Column
import androidx.compose.foundation.layout.Row
import androidx.compose.foundation.layout.fillMaxSize
import androidx.compose.foundation.layout.fillMaxWidth
import androidx.compose.foundation.layout.padding
import androidx.compose.foundation.rememberScrollState
import androidx.compose.foundation.verticalScroll
import androidx.compose.foundation.text.KeyboardOptions
import androidx.compose.material.icons.Icons
import androidx.compose.material.icons.filled.Info
import androidx.compose.material3.AlertDialog
import androidx.compose.material3.DropdownMenuItem
import androidx.compose.material3.ExperimentalMaterial3Api
import androidx.compose.material3.ExposedDropdownMenuBox
import androidx.compose.material3.ExposedDropdownMenuDefaults
import androidx.compose.material3.Icon
import androidx.compose.material3.OutlinedTextField
import androidx.compose.material3.Switch
import androidx.compose.material3.Text
import androidx.compose.material3.TextButton
import androidx.compose.runtime.Composable
import androidx.compose.runtime.collectAsState
import androidx.compose.runtime.getValue
import androidx.compose.runtime.mutableIntStateOf
import androidx.compose.runtime.mutableLongStateOf
import androidx.compose.runtime.mutableStateOf
import androidx.compose.runtime.remember
import androidx.compose.runtime.setValue
import androidx.compose.ui.Alignment
import androidx.compose.ui.Modifier
import androidx.compose.ui.text.input.KeyboardType
import androidx.compose.ui.unit.dp
import androidx.lifecycle.viewmodel.compose.viewModel
import at.xaxa.ledger.data.Entry
import at.xaxa.ledger.ui.AppViewModelProvider
import at.xaxa.ledger.ui.ButtonSuccess
import at.xaxa.ledger.ui.DatePickerDocked
import at.xaxa.ledger.ui.category.iconNames
import at.xaxa.ledger.ui.category.icons
import kotlin.math.abs
import kotlin.math.log

@OptIn(ExperimentalMaterial3Api::class)
@Composable
fun Add(
    modifier: Modifier = Modifier,
    onCardClick: () -> Unit,
    addViewModel: AddViewModel = viewModel(factory = AppViewModelProvider.Factory)
) {
    var name by remember { mutableStateOf("") }
    var categories = addViewModel.categoryUiState.categories.collectAsState(emptyList()).value

    var spending by remember { mutableStateOf("") }
    var selectedDate by remember { mutableLongStateOf(System.currentTimeMillis()) }
    var expanded by remember { mutableStateOf(false) }
    var selectedIconIndex by remember { mutableStateOf(0) }
    var selectedCategoryName by remember { mutableStateOf("") }
    var selectedCategory by remember { mutableIntStateOf(-1) }
    var parsingError by remember { mutableStateOf(false) }
    var isIncome by remember { mutableStateOf(true) } // true for Income, false for Expense

    if (parsingError) {
        AlertDialog(
            icon = {
                Icon(Icons.Default.Info, contentDescription = "Info Icon")
            },
            title = {
                Text(text = "Parsing error")
            },
            text = {
                Text(text = "One or more fields contain invalid input. Please review your entries and try again.")
            },
            onDismissRequest = {
                parsingError = false
            },
            confirmButton = {
                TextButton(
                    onClick = {
                        parsingError = false
                    }
                ) {
                    Text("Okay")
                }
            }
        )
    }

    Column(
        modifier = modifier
            .fillMaxSize()
            .padding(16.dp, 0.dp),
        horizontalAlignment = Alignment.CenterHorizontally
    ) {
        Column(
            Modifier.weight(1f)
        ) {
            OutlinedTextField(
                value = name,
                onValueChange = { name = it },
                maxLines = 3,
                label = { Text("Name") },
                modifier = Modifier.fillMaxWidth().verticalScroll(rememberScrollState()),
            )

            // Toggle between Income and Expense


            OutlinedTextField(
                value = spending,
                onValueChange = {
                    val isValidSpending = it.matches(Regex("^-?(\\d{0,7})(\\.\\d{0,2})?$"))
                    if (isValidSpending) {
                        spending = it
                    }
                },
                label = { Text("Amount") },
                modifier = Modifier
                    .fillMaxWidth(),
                keyboardOptions = KeyboardOptions.Default.copy(keyboardType = KeyboardType.Number),
            )

            ExposedDropdownMenuBox(
                expanded = expanded,
                onExpandedChange = { expanded = it }
            ) {
                OutlinedTextField(
                    value = selectedCategoryName,
                    onValueChange = {},
                    label = { Text("Category") },
                    readOnly = true,
                    leadingIcon = {
                        Icon(
                            imageVector = icons[selectedIconIndex],
                            contentDescription = "Leading Icon"
                        )
                    },
                    trailingIcon = {
                        ExposedDropdownMenuDefaults.TrailingIcon(expanded = expanded)
                    },
                    modifier = Modifier
                        .menuAnchor()
                        .fillMaxWidth()
                )

                ExposedDropdownMenu(
                    expanded = expanded,
                    onDismissRequest = { expanded = false }
                ) {
                    categories.forEachIndexed { index, category ->
                        DropdownMenuItem(
                            text = { Text(text = category.categoryName) },
                            onClick = {
                                selectedIconIndex = category.icon
                                expanded = false
                                selectedCategoryName = category.categoryName
                                selectedCategory = category._id
                            },
                            leadingIcon = {
                                androidx.compose.material3.Icon(
                                    imageVector = icons[category.icon],
                                    contentDescription = iconNames[category.icon]
                                )
                            }
                        )
                    }
                }
            }

            DatePickerDocked { dateMilis ->
                selectedDate = dateMilis
            }
            Row(
                modifier = Modifier
                    .fillMaxWidth()
                    .padding(8.dp),
                verticalAlignment = Alignment.CenterVertically
            ) {
                Text(text = if (isIncome) "Income" else "Expense")
                Switch(
                    modifier = Modifier.padding(8.dp, 0.dp),
                    checked = isIncome,
                    onCheckedChange = {
                        isIncome = it
                        if(isIncome){
                            spending =  abs(spending.toFloat()).toString()
                        }else{
                            spending =  (abs(spending.toFloat())*-1f).toString()
                        }
                    }
                )
            }
        }


        Box(
            modifier = Modifier
                .fillMaxWidth(),
            contentAlignment = Alignment.Center
        ) {
            ButtonSuccess(
                modifier = Modifier,
                "Add Transaction",
                onClick = {
                    val isValidSpending = spending.matches(Regex("^-?\\d*(\\.\\d{0,2})?$"))
                    if (name.isNotBlank() && isValidSpending && selectedDate != 0L && selectedCategory != -1) {
                        val newEntry = Entry(
                            id = 0,
                            name = name,
                            amount = spending.toFloat(),
                            date = selectedDate,
                            categoryID = selectedCategory
                        )
                        addViewModel.addEntryToDB(newEntry)
                        onCardClick()
                    } else {
                        parsingError = true
                    }
                }
            )
        }
    }
}